@@ -134,17 +134,15 @@ def parse_args():
         '-r', '--release', help="The Fedora release to search",
         type=_release, required=False, metavar="1-99 or Rawhide")
     parser_images.add_argument(
-        '-m', '--milestone', help="A milestone to search (e.g. Alpha or "
-        "Beta). 'Final' and 'RC' are synonyms. 'Atomic' is for two-week "
-        "nightly Atomic composes. 'Docker' is for nightly stable Docker "
-        "composes. 'Branched' and 'Rawhide' are the two mainline nightly "
-        "compose types, see the wiki for more details. 'Production' is for "
-        "candidate composes on the compose server (kojipkgs) and identified "
-        "by a date and respin, as opposed to the same composes synced to alt "
-        "and identified by label. Respin is for the semi-official post-"
-        "release live respin composes.",
-        choices=['Alpha', 'Beta', 'Final', 'RC', 'Atomic', 'Docker',
-                 'Branched', 'Rawhide', 'Production', 'Respin'])
+        '-m', '--milestone', help="A milestone to search (e.g. Alpha or Beta). 'Final' and 'RC' "
+        "are synonyms. 'Atomic' is for two-week nightly Atomic composes. 'Docker' is for nightly "
+        "stable Docker composes. 'Cloud' is for nightly stable Cloud composes. 'Branched' and "
+        "'Rawhide' are the two mainline nightly compose types, see the wiki for more details. "
+        "'Production' is for candidate composes on the compose server (kojipkgs) and identified "
+        "by a date and respin, as opposed to the same composes synced to alt and identified by "
+        "label. Respin is for the semi-official post-release live respin composes.",
+        choices=['Alpha', 'Beta', 'Final', 'RC', 'Atomic', 'Docker', 'Cloud', 'Branched', 'Rawhide',
+                 'Production', 'Respin'])
     parser_images.add_argument(
         '-c', '--compose', '--date', help="A compose or date to "
         "search, e.g. 1 or 20160314. You may also pass a compose and respin "
@@ -208,3 +206,5 @@ def main():
 
 if __name__ == '__main__':
     main()
+
+# vim: set textwidth=100 ts=8 et sw=4:
