@@ -50,7 +50,14 @@ HTTPS_DL = 'https://dl.fedoraproject.org/pub'
 NIGHTLY_BASE = 'https://kojipkgs.fedoraproject.org/compose'
 KNOWN_PREFIXES = (HTTPS, 'http://download.fedoraproject.org/pub',
                   HTTPS_DL, 'http://dl.fedoraproject.org/pub',
-                  NIGHTLY_BASE, 'http://kojipkgs.fedoraproject.org/compose')
+                  NIGHTLY_BASE, 'http://kojipkgs.fedoraproject.org/compose',
+                  # secondary arch compose locations, see RHBZ #1435953
+                  'http://ppc.koji.fedoraproject.org/compose',
+                  'https://ppc.koji.fedoraproject.org/compose',
+                  'http://s390.koji.fedoraproject.org/compose',
+                  'https://s390.koji.fedoraproject.org/compose',
+                  'http://arm.koji.fedoraproject.org/compose',
+                  'https://arm.koji.fedoraproject.org/compose',)
 PDC_API = 'https://pdc.fedoraproject.org/rest_api/v1'
 
 # we could import these from pungi but jeez, the dependencies are
@@ -91,7 +98,9 @@ SUBVARIANTS = (
     LoadTuple('Docker_Base', ('docker-base',)),
     LoadTuple('Workstation', ('workstation', 'work')),
     LoadTuple('Server', ('server',)),
-    LoadTuple('Everything', ('everything',))
+    LoadTuple('Everything', ('everything',)),
+    LoadTuple('Python_Classroom', ('python-classroom',)),
+    LoadTuple('LXQt', ('lxqt',)),
 )
 
 # ARCHES
@@ -108,3 +117,5 @@ ARCHES = (
 
 RESPINRE = re.compile(r'F(?P<release>\d+)-(?P<subv>[^-]+)-(?P<arch>[^-]+)-(?P<date>\d{8,8})\.iso',
                       re.I)
+
+# vim: set textwidth=100 ts=8 et sw=4:
