@@ -103,8 +103,9 @@ def urlopen_retries(url, tries=5):
     while not resp and tries:
         try:
             resp = urlopen(url)
-        except (ValueError, URLError, HTTPError):
+        except (ValueError, URLError, HTTPError) as err:
             logger.debug("HTTP error! Retrying...")
+            logger.debug("Error: %s", err)
             tries -= 1
     if not resp:
         raise ValueError("urlopen_retries: Failed to open {0}!".format(url))
@@ -321,7 +322,8 @@ def pdc_query(path, params=None):
         params = list(params.items())
     except AttributeError:
         pass
-    baseurl = '/'.join((fedfind.const.PDC_API, path.strip('/')))
+    baseurl = '/'.join((fedfind.const.PDC_API, path.strip('/'))) + '/'
+    logger.debug("pdc_query: baseurl is: %s", baseurl)
     # cache key is the URL of the query, but without the page param
     cachekey = '?'.join((baseurl, urlencode(params)))
 
@@ -379,6 +381,11 @@ def pdc_query(path, params=None):
 
     # write cache if we decided to
     if writecache:
+        # I've seen CACHEDIR suddenly vanish, when it's a temp dir; let's
+        # check and re-create it if necessary
+        if not os.path.exists(CACHEDIR):
+            os.makedirs(CACHEDIR)
+
         with codecs.open(cachefile, 'w', encoding='utf-8') as cachefh:
             cachefh.write(json.dumps(cache, separators=(',', ':'), indent=0))
 
@@ -396,14 +403,14 @@ def find_cid(string):
     # must start with 'Fedora', but can have more text after that (and
     # can contain dashes, because life is awful; in the rest of the ID
     # dashes can only be separators).
-    cidpatt = re.compile(r'Fedora.*-[^-]+-\d{8,8}(\.\w)?\.\d+')
+    cidpatt = re.compile(r'(Fedora|FACD).*-[^-]+-\d{8,8}(\.\w)?\.\d+')
     match = cidpatt.search(string)
     if match:
         return match.group(0)
     else:
         return ''
 
-def parse_cid(origcid, dist=False):
+def parse_cid(origcid, dist=False, dic=False):
     """If dist is false-y, get (release, date, type, respin) values
     from a Pungi 4 compose id. If dist is truth-y, get (dist, release,
     date, type, respin) values from a Pungi 4 compose id.
@@ -413,32 +420,141 @@ def parse_cid(origcid, dist=False):
     if not cid:
         raise ValueError("{0} does not appear to be a valid Pungi 4 Fedora "
                          "compose ID!".format(origcid))
-    # The dist can have '-' in it, of course, because life is awful
-    # FIXME: This is very stupid and will break on layered products,
-    # which have a lot more - in them. It can theoretically break in
-    # several other cases, but I don't know if Fedora is currently
-    # hitting any of those. I'm currently thinking about how to deal
-    # with this. See:
+    # Writing a truly generic compose ID parser is pretty tricky. See
     # https://github.com/release-engineering/productmd/issues/77
-    # We may just have to rejig things to require the full compose
-    # metadata and give up on parsing compose IDs at all. That would
-    # be a lot of work for fedfind. A somewhat uglier alternative for
-    # fedfind's purposes would be to give it special knowledge of the
-    # Fedora product shortnames with - in them, or a generic handler
-    # for 'Fedora-(FOO)' or something.
-    (gotdist, release, _) = cid.rsplit('-', 2)
-    release = release.lower()
-    (date, typ, respin) = productmd.composeinfo.get_date_type_respin(cid)
-    if dist:
-        return (gotdist, release, date, typ, respin)
-    else:
-        return (release, date, typ, respin)
+    # for some background. This is my best effort. I'm trying to get
+    # it upstreamed currently.
 
-def cid_from_label(release, label):
+    # init values
+    mainshort = ''
+    mainver = ''
+    maintype = ''
+    baseshort = ''
+    basever = ''
+    basetype = ''
+    variant = ''
+
+    # find date, type, respin
+    (date, typ, respin) = productmd.composeinfo.get_date_type_respin(cid)
+    # now split on the date, we only care about what comes before
+    part = cid.split(date)[0][:-1]
+
+    # Handle "HACK: there are 2 RHEL 5 composes"
+    if part.endswith("-Client"):
+        variant = "Client"
+        part = part[:-len('-Client')]
+    elif part.endswith("-Server"):
+        variant = "Server"
+        part = part[:-len('-Server')]
+
+    # Next part back must be either a version type suffix or a version
+    # we don't know yet if this is the main version or the base
+    # version for a layered product
+    sometype = ''
+    # copy the list of release types and sort it in descending order
+    # by length, so we won't match on a subset of the entire type.
+    _reltypes = list(productmd.common.RELEASE_TYPES)
+    # updates-testing was only added in productmd 1.9, which is only
+    # in Rawhide and F27 u-t at time of writing (2017-10-27). This can
+    # be removed when 1.9 is stable in all releases
+    if 'updates-testing' not in _reltypes:
+        _reltypes.append('updates-testing')
+    _reltypes.sort(key=len, reverse=True)
+    for _reltype in _reltypes:
+        # if part ends in _reltype, take it as 'sometype' and cut it
+        if part.lower().endswith(_reltype.lower()):
+            sometype = _reltype
+            part = part[:-len(_reltype)-1]
+            break
+    # now whatever's at the end of 'part' must be a version
+    somever = part.split('-')[-1]
+    # strip it off
+    part = '-'.join(part.split('-')[:-1])
+
+    # what remains is either:
+    # mainshort
+    # or:
+    # mainshort-mainver(-maintype)-baseshort
+    # But this is where things get fun, because sadly, both mainshort
+    # and baseshort could have - in them and mainver could have a type
+    # suffix. So, life is fun. Working in our favour, shortnames are
+    # *not* allowed to contain digits. Let's see if we can spot what
+    # looks like a '-mainver'- component...
+    elems = part.split('-')
+    for (idx, cand) in enumerate(elems):
+        # can't be the first or the last
+        if idx == 0 or idx == len(elems) - 1:
+            continue
+        # now see if the cand looks like a version
+        # FIXME: we should use RELEASE_VERSION_RE here, but not until
+        # it enforces this stricter rule. We have decided that only
+        # 'Rawhide' or anything that looks like a version number -
+        # starts with a digit, can have any number of subsequent
+        # digits and/or digit groups separated with single periods -
+        # is a valid version.
+        verreg = re.compile(r'(Rawhide|Bikeshed|[0-9]+(\.?[0-9]+)*)$')
+        match = verreg.match(cand)
+        if match:
+            mainver = match.group(1)
+            # check for a type suffix. of course, because life is
+            # awful, some of these have dashes in them, so we get to
+            # be careful...
+            for _reltype in _reltypes:
+                rtelems = _reltype.lower().split('-')
+                if elems[idx+1:idx+1+len(rtelems)] == rtelems:
+                    maintype = _reltype
+                    # pop out the right number of elems
+                    for i in range(0,len(rtelems)):
+                        elems.pop(idx+1)
+                    break
+            basever = somever
+            basetype = sometype
+            mainshort = '-'.join(elems[:idx])
+            baseshort = '-'.join(elems[idx+1:])
+            break
+
+    # if we didn't establish a mainver above, we must not be layered,
+    # and what remains is just mainshort, and somever is mainver
+    if not mainshort:
+        mainshort = '-'.join(elems)
+        mainver = somever
+        maintype = sometype
+
+    if not maintype:
+        maintype = 'ga'
+    if basever and not basetype:
+        basetype = 'ga'
+
+    if dic:
+        return {
+            'short': mainshort,
+            'version': mainver,
+            'version_type': maintype,
+            'base_short': baseshort,
+            'base_version': basever,
+            'base_type': basetype,
+            'variant': variant,
+            'date': date,
+            'compose_type': typ,
+            'respin': respin
+        }
+    elif dist:
+        return (mainshort, mainver.lower(), date, typ, respin)
+    else:
+        return (mainver.lower(), date, typ, respin)
+
+def cid_from_label(release, label, short='fedora'):
     """Get the compose ID for a compose by label. Must also specify
-    the release as labels are re-used per release. Uses PDC.
+    the release as labels are re-used per release. Can specify a
+    shortname, because not just 'Fedora' composes have labels; the
+    stable release nightly composes (Cloud, Atomic, Docker) also do.
+    We can't tell from the label itself what the shortname might be,
+    in fact each day we should get three composes with identical
+    labels, so there's really no option but to require the user to
+    specify. Uses PDC.
     """
-    params = {'release': "fedora-{0}".format(release), 'compose_label': label}
+    short = short.lower()
+    params = {'release': "{0}-{1}".format(short, release), 'compose_label': label}
     res = pdc_query('composes', params)
     if res:
         return res[0]['compose_id']
@@ -449,7 +565,7 @@ def label_from_cid(cid):
     production composes have discoverable labels; will return the
     empty string for other composes.
     """
-    typ = parse_cid(cid)[2]
+    typ = parse_cid(cid, dic=True)['compose_type']
     if typ != 'production':
         return ''
     params = {'compose_id': cid}
@@ -625,6 +741,12 @@ def _get_filelist(flist='fedora', record=None):
 
     # If we got here, either we don't have a cache file at all or the
     # dates don't match. So we need to re-download it
+
+    # I've seen CACHEDIR suddenly vanish, when it's a temp dir; let's
+    # check and re-create it if necessary
+    if not os.path.exists(CACHEDIR):
+        os.makedirs(CACHEDIR)
+
     with codecs.open(cachefile + '.tmp', 'w', encoding='utf-8') as cachefh:
         # Write the modified date line
         cachefh.write(modified + '\n')
@@ -645,3 +767,5 @@ def _get_filelist(flist='fedora', record=None):
     cachefh = codecs.open(cachefile, 'r', encoding='utf-8')
     next(cachefh)
     return cachefh
+
+# vim: set textwidth=100 ts=8 et sw=4:
