@@ -64,19 +64,19 @@ def get_release_cid(cid):
     return get_release(cid=cid)
 
 def get_release(release='', milestone='', compose='', respin=None,
-                cid='', label='', url='', promote=False):
+                cid='', label='', url='', promote=False, dist='Fedora'):
     """Function to return an appropriate Release subclass for the
     version information. Figures out if you want a nightly, compose,
     milestone, or stable release and gives it to you. Catches some
     nonsense choices. Tries to figure out the appropriate subclass
     (current, archive, core...) for stable releases. Can work off
-    fedfind/wikitcms-style 'release, milestone, compose, respin',
+    fedfind/wikitcms-style 'release, milestone, compose, respin, dist',
     or from a compose URL (expected to be the URL available from
     releng fedmsgs - the /compose directory of a Pungi 4 compose),
     or from a Pungi 4 compose ID, or from a Pungi 4 compose label.
     Yeah, that's a lot of choice.
 
-    Priority goes to url, then label, then cid, then r/m/c/r. So if
+    Priority goes to url, then label, then cid, then r/m/c/r/d. So if
     url is specified, all other values are ignored; if url is not
     specified but label is, label is used and all other values except
     release are ignored; if neither url nor label is specified but
@@ -103,7 +103,7 @@ def get_release(release='', milestone='', compose='', respin=None,
     plausible release. Of course this function may also return a
     release that does not exist, and this is desired in some cases.
 
-    Note that if all four values are left unset, you will get the
+    Note that if all version values are left unset, you will get the
     current date's most recent RawhideNightly if there is one, or
     else you will get a non-existent RawhideNightly for today's date
     with respin 0. Also note that either release or milestone being
@@ -115,14 +115,27 @@ def get_release(release='', milestone='', compose='', respin=None,
     fedfind and wikitcms versions are transparently interchangeable.
 
     For nightlies, pass compose as 'YYYYMMDD'. You'll wind up with
-    either a RawhideNightly, BranchedNightly or Atomic, depending on
-    release and milestone. If you set a valid release number and leave
-    milestone unset, you'll get an Atomic only if it exists and
-    BranchedNightly for the same values doesn't, otherwise you'll get
-    BranchedNightly. You can never get a Docker by nightly detection,
-    you must specify it via milestone, cid or url. If you set a valid
-    release number and a valid nightly milestone, you'll get what you
-    asked for. For any other case you'll wind up with RawhideNightly.
+    either a Rawhide, Branched, Atomic, Docker or Cloud nightly,
+    depending on release and milestone. If you set a valid release
+    number and leave milestone unset, you'll get an Atomic only if it
+    exists and BranchedNightly for the same values doesn't, otherwise
+    you'll get BranchedNightly. You can never get a Docker or Cloud
+    by nightly detection, you must specify it via milestone, cid or
+    url. If you set a valid release number and a valid nightly
+    milestone, you'll get what you asked for. For any other case
+    you'll wind up with RawhideNightly (or RawhideModularNightly).
+
+    At present the only interesting values for dist are 'Fedora' (the
+    default) and 'Fedora-Modular'. If you set it to 'Fedora-Modular',
+    you can get a RawhideModularNightly where you would otherwise get
+    a RawhideNightly, or a BranchedModularNightly where you would
+    otherwise get a BranchedNightly. These are separate composes built
+    from 'Modularity' modules derived from the Rawhide and Branched
+    branches, using 'Fedora-Modular' as their 'dist' (as fedfind calls
+    it) / 'shortname' (as productmd calls it). There might possibly be
+    candidate and milestone 'Fedora-Modular' composes too, but there
+    haven't been any *yet* and this code does not yet support them.
+
     If you set a nightly type milestone and leave compose unset, we
     will use today's date as the compose. For all cases, if you
     specify respin, you'll get an instance with that respin, whether
@@ -179,7 +192,7 @@ def get_release(release='', milestone='', compose='', respin=None,
         otherwise we leave it blank (this is so e.g. (22, '', '')
         returns Fedora 22).
         """
-        snaps = ('rawhide', 'branched', 'atomic', 'docker', 'production')
+        snaps = ('rawhide', 'bikeshed', 'branched', 'atomic', 'docker', 'cloud', 'production')
         if not compose and not_or_str((release, milestone), snaps):
             logger.debug("Guessing date")
             compose = fedfind.helpers.date_check(
@@ -201,17 +214,19 @@ def get_release(release='', milestone='', compose='', respin=None,
         # it.
         if release:
             check = str(release).lower()
-            if check.isdigit() or check == 'rawhide':
+            if check.isdigit() or check in ('rawhide', 'bikeshed'):
                 return release
             else:
                 raise ValueError("get_release(): Release must be a number "
-                                 "or Rawhide!")
+                                 "or Rawhide or Bikeshed!")
 
         # If we were specifically asked for a valid milestone, return
         # appropriate release.
         if milestone.lower() in ('branched', 'production'):
             return fedfind.helpers.get_current_release() + 1
-        elif milestone.lower() in ('atomic', 'docker'):
+        elif milestone.lower() in ('atomic', 'docker', 'cloud'):
+            # Note: cloud is actually composed for both current and
+            # prev, but we gotta guess something...
             return fedfind.helpers.get_current_release()
         elif milestone.lower() == 'respin':
             # no expected release here
@@ -254,19 +269,29 @@ def get_release(release='', milestone='', compose='', respin=None,
             rel = CurrentRelease(release)
         return rel
 
-    def guess_nightly_respin(release, milestone, date, cid=None):
+    def guess_nightly_respin(dist, release, milestone, date, cid=None):
         """We have to handle 'respins' for nightlies if one wasn't
         explicitly passed. We just count down from 5 and stop when one
         exists or we hit 0.
         """
         if release == 'Rawhide':
-            testcls = partial(RawhideNightly, compose=date)
+            if dist.lower() == 'fedora-modular':
+               testcls = partial(RawhideModularNightly, compose=date)
+            else:
+                testcls = partial(RawhideNightly, compose=date)
+        elif release == 'Bikeshed' and dist.lower() == 'fedora-modular':
+            testcls = partial(BikeshedModularNightly, compose=date)
         elif milestone.lower() == 'atomic':
             testcls = partial(AtomicNightly, release=release, compose=date)
         elif milestone.lower() == 'docker':
             testcls = partial(DockerNightly, release=release, compose=date)
+        elif milestone.lower() == 'cloud':
+            testcls = partial(CloudNightly, release=release, compose=date)
         else:
-            testcls = partial(BranchedNightly, release=release, compose=date)
+            if dist.lower() == 'fedora-modular':
+                testcls = partial(BranchedModularNightly, release=release, compose=date)
+            else:
+                testcls = partial(BranchedNightly, release=release, compose=date)
         respin = 5
         while respin > -1:
             test = testcls(respin=respin, cid=cid)
@@ -275,41 +300,52 @@ def get_release(release='', milestone='', compose='', respin=None,
             respin -= 1
         return test
 
-    def get_nightly(release, milestone, date, respin=None, cid=None):
+    def get_nightly(dist, release, milestone, date, respin=None, cid=None):
         """Return the appropriate nightly release class for a given
-        release, milestone, date, and respin. Handles guessing the
+        dist, release, milestone, date, and respin. Handles guessing
         milestone. If respin is not specified, handles guessing that.
         """
         if milestone.lower() == 'production':
             if respin is None:
                 raise ValueError("Cannot guess respin for a production "
                                  "candidate compose!")
+            if dist.lower() == 'fedora-modular':
+                return ModularProduction(release, date, respin, cid=cid)
             return Production(release, date, respin, cid=cid)
 
-        if milestone.lower() in ('atomic', 'docker', 'branched'):
+        if milestone.lower() in ('atomic', 'docker', 'cloud', 'branched'):
             if milestone.lower() == 'atomic':
                 klass = AtomicNightly
             elif milestone.lower() == 'docker':
                 klass = DockerNightly
+            elif milestone.lower() == 'cloud':
+                klass = CloudNightly
             else:
-                klass = BranchedNightly
+                if dist.lower() == 'fedora-modular':
+                    klass = BranchedModularNightly
+                else:
+                    klass = BranchedNightly
             if respin is not None:
                 return klass(
                     release=release, compose=date, respin=respin, cid=cid)
             else:
-                return guess_nightly_respin(release, milestone, date, cid=cid)
+                return guess_nightly_respin(dist, release, milestone, date, cid=cid)
 
         else:
             if milestone:
                 logger.warn("Invalid milestone %s for nightly! Ignoring.",
                             milestone)
             # Guess: try Branched, if it doesn't exist, try Atomic. We
-            # don't include 'production' or Docker in guessing.
+            # don't include 'production' or Docker or Cloud.
             if respin is not None:
-                branched = BranchedNightly(release=release, compose=date,
-                                           respin=respin, cid=cid)
+                if dist.lower() == 'fedora-modular':
+                    branched = BranchedModularNightly(release=release, compose=date,
+                                               respin=respin, cid=cid)
+                else:
+                    branched = BranchedNightly(release=release, compose=date,
+                                               respin=respin, cid=cid)
             else:
-                branched = guess_nightly_respin(release, 'branched', date, cid=cid)
+                branched = guess_nightly_respin(dist, release, 'branched', date, cid=cid)
             if branched.exists:
                 return branched
             else:
@@ -317,7 +353,7 @@ def get_release(release='', milestone='', compose='', respin=None,
                     atomic = AtomicNightly(
                         release=release, compose=date, respin=respin, cid=cid)
                 else:
-                    atomic = guess_nightly_respin(release, 'atomic', date, cid=cid)
+                    atomic = guess_nightly_respin(dist, release, 'atomic', date, cid=cid)
                 if atomic.exists:
                     return atomic
                 return branched
@@ -329,41 +365,51 @@ def get_release(release='', milestone='', compose='', respin=None,
         return (milestone, compose, int(respin))
 
     def parse_cid(cid):
-        """Produce a release/milestone/compose/respin tuple from a
-        compose ID. Will always give the values that produce a raw,
+        """Produce a dist/release/milestone/compose/respin tuple from
+        a compose ID. Will always give the values that produce a raw,
         non-mirrored Release instance. Should work for fedfind fake
         CIDs (returning a Release class instance matching the one that
         # produced the fake CID in the first place).
         """
         # Assume a real Pungi 4 compose ID
-        (dist, release, date, typ, respin) = fedfind.helpers.parse_cid(cid, dist=True)
+        ciddic = fedfind.helpers.parse_cid(cid, dic=True)
+        (dist, release, date, typ, respin) = (ciddic['short'], ciddic['version'], ciddic['date'],
+                                              ciddic['compose_type'], ciddic['respin'])
+        release = release.capitalize()
 
-        # Atomic stable nightly composes, 'Fedora-Atomic' product name
+        if ciddic['version_type'] in ('updates', 'updates-testing') and dist != "FedoraRespin":
+            raise ValueError("get_release(): fedfind does not support updates or updates-testing "
+                             "composes like {0} as they contain no images".format(cid))
+
+        # Atomic stable nightly composes, 'Fedora-Atomic' shortname
         if dist == 'Fedora-Atomic':
-            return (release, 'Atomic', date, int(respin))
+            return (dist, release, 'Atomic', date, int(respin))
 
-        # Docker stable nightly composes, 'Fedora-Docker' product name
+        # Docker stable nightly composes, 'Fedora-Docker' shortname
         if dist == 'Fedora-Docker':
-            return (release, 'Docker', date, int(respin))
+            return (dist, release, 'Docker', date, int(respin))
+
+        # Cloud stable nightly composes, 'Fedora-Cloud' shortname
+        if dist == 'Fedora-Cloud':
+            return (dist, release, 'Cloud', date, int(respin))
 
         # Semi-official stable live respin composes
         if dist == 'FedoraRespin':
-            return (release, 'Respin', date, 0)
+            return (dist, release, 'Respin', date, 0)
 
         if typ == 'nightly':
-            if release == 'rawhide':
+            if release == 'Rawhide':
                 milestone = ''
-                release = 'Rawhide'
             else:
                 milestone = 'Branched'
-            return (release, milestone, date, int(respin))
+            return (dist, release, milestone, date, int(respin))
         elif typ == 'production':
             # date 19700101 indicates a fedfind fake CID, most likely
             # for a stable release, so just return the release number
             # so we get a stable release class
             if date == '19700101':
-                return (release, '', '', 0)
-            return (release, 'Production', date, int(respin))
+                return (dist, release, '', '', 0)
+            return (dist, release, 'Production', date, int(respin))
         else:
             raise ValueError("get_release(): could not parse compose ID "
                              "{0}!".format(cid))
@@ -371,13 +417,13 @@ def get_release(release='', milestone='', compose='', respin=None,
     def parse_url(url):
         """Handle a URL if specified. If this looks like the URL for
         RespinRelease, return that. Otherwise if we can find a compose
-        ID, return appropriate release/milestone/compose/respin set.
-        Otherwise just return None (we don't handle URLs for things
-        like stable releases ATM).
+        ID, return appropriate dist/release/milestone/compose/respin
+        tuple. Otherwise just return None (we don't handle URLs for
+        things like stable releases ATM).
         """
         if url.rstrip('/').endswith('/alt/live-respins'):
             # special case: assume we wanted RespinRelease
-            return ('', 'Respin', '', 0, None)
+            return ('Fedora', '', 'Respin', '', 0, None)
         cid = fedfind.helpers.find_cid(url)
         if cid:
             try:
@@ -391,7 +437,7 @@ def get_release(release='', milestone='', compose='', respin=None,
     if url:
         ret = parse_url(url)
         if ret:
-            (release, milestone, compose, respin, cid) = ret
+            (dist, release, milestone, compose, respin, cid) = ret
         else:
             # we couldn't parse the URL, so just give a raw Pungi4
             # Release and hope for the best...but let's only do it
@@ -402,7 +448,7 @@ def get_release(release='', milestone='', compose='', respin=None,
                 raise ValueError("Unexpected URL {0}, expected to start with one of {1}".format(
                     url, ' '.join(fedfind.const.KNOWN_PREFIXES)))
     elif cid:
-        (release, milestone, compose, respin) = parse_cid(cid)
+        (dist, release, milestone, compose, respin) = parse_cid(cid)
         if promote:
             label = fedfind.helpers.label_from_cid(cid)
             if label:
@@ -444,14 +490,25 @@ def get_release(release='', milestone='', compose='', respin=None,
             raise ValueError("get_release(): for Rawhide, compose must be "
                              "a date!")
         if respin is not None:
-            return RawhideNightly(compose, respin, cid=cid)
+            if dist.lower() == 'fedora-modular':
+                return RawhideModularNightly(compose, respin, cid=cid)
+            else:
+                return RawhideNightly(compose, respin, cid=cid)
         else:
-            return guess_nightly_respin(release, milestone, compose, cid=cid)
+            return guess_nightly_respin(dist, release, milestone, compose, cid=cid)
+
+    # 'Bikeshed' modular nightly case (this is basically Rawhide, but
+    # releng decided to change the name for some goddamn reason).
+    if release == 'Bikeshed' and dist.lower() == 'fedora-modular':
+        if respin is not None:
+            return BikeshedModularNightly(compose, respin, cid=cid)
+        else:
+            return guess_nightly_respin(dist, release, milestone, compose, cid=cid)
 
     # All nightly cases. 'Production' also handled here, as 'compose'
     # is a date.
     if fedfind.helpers.date_check(compose, fail_raise=False):
-        return get_nightly(release, milestone, compose, respin, cid=cid)
+        return get_nightly(dist, release, milestone, compose, respin, cid=cid)
 
     # Production candidates ('Alpha 1.1' etc.)
     if milestone and compose:
@@ -460,7 +517,10 @@ def get_release(release='', milestone='', compose='', respin=None,
                              "must be specified or included in compose")
         # first, see if we have a Compose - i.e. the compose has been
         # synced to alt/stage
-        rel = Compose(release, milestone, compose, respin, cid=cid)
+        if dist.lower() == 'fedora-modular':
+            rel = ModularCompose(release, milestone, compose, respin, cid=cid)
+        else:
+            rel = Compose(release, milestone, compose, respin, cid=cid)
         if rel.exists:
             return rel
 
@@ -476,7 +536,7 @@ def get_release(release='', milestone='', compose='', respin=None,
             # so not found as a Compose above. Re-use it in this case.
             cid = fedfind.helpers.cid_from_label(release, label)
         if cid:
-            (release, milestone, compose, respin) = parse_cid(cid)
+            (dist, release, milestone, compose, respin) = parse_cid(cid)
             return Production(release, compose, respin, cid=cid)
         else:
             raise ValueError("get_release(): could not find release for {0}, "
@@ -909,7 +969,7 @@ class Pungi4Release(Release):
         """
         if self._release:
             return self._release
-        return fedfind.helpers.parse_cid(self.cid)[0]
+        return fedfind.helpers.parse_cid(self.cid, dic=True)['version'].lower()
 
     @property
     def milestone(self):
@@ -983,9 +1043,11 @@ class Pungi4Release(Release):
     @property
     def product(self):
         """Compose "product" - the first component of the compose ID,
-        so 'Fedora' or 'Fedora-Atomic' or 'Fedora-Docker'.
+        so 'Fedora', 'Fedora-Atomic', 'Fedora-Docker', 'Fedora-Cloud'
+        etc. In productmd terms this is in fact the 'short name', but
+        Fedora releng decided not to make them very short...
         """
-        return fedfind.helpers.parse_cid(self.cid, dist=True)[0]
+        return fedfind.helpers.parse_cid(self.cid, dic=True)['short']
 
     @property
     def previous_release(self):
@@ -1064,6 +1126,60 @@ class RawhideNightly(Pungi4Release):
         return tuple(imgs)
 
 
+class RawhideModularNightly(Pungi4Release):
+    """Rawhide modular "nightly" (bit of a misnomer, now) composes.
+    Note these were replaced by BikeshedModularNightly between
+    20171003 and 20171013, there are none of these composes after
+    that date; once the earlier composes are removed this class will
+    likely be turned into a redirect to BikeshedModularNightly.
+    """
+    def __init__(self, compose, respin=0, cid=None):
+        compose = str(compose)
+        respin = str(respin)
+        path = "Fedora-Modular-Rawhide-{0}.n.{1}/compose".format(
+            compose, respin)
+        url = '/'.join((fedfind.const.NIGHTLY_BASE, path))
+        super(RawhideModularNightly, self).__init__(
+            url, release="Rawhide", milestone='', compose=compose,
+            respin=respin, typ="nightly", cid=cid)
+
+    @property
+    def expected_images(self):
+        return (
+            ('docker_base', 'docker', 'x86_64'),
+            ('server', 'dvd', 'arm'),
+            ('server', 'dvd', 'x86_64'),
+            ('server', 'qcow2', 'x86_64'),
+            ('server', 'raw-xz', 'x86_64'),
+        )
+
+
+class BikeshedModularNightly(Pungi4Release):
+    """Bikeshed modular "nightly" (bit of a misnomer, now) composes.
+    This is basically Rawhide for modular, I don't know why releng
+    decided to change the name other than to screw with me...
+    """
+    def __init__(self, compose, respin=0, cid=None):
+        compose = str(compose)
+        respin = str(respin)
+        path = "Fedora-Modular-Bikeshed-{0}.n.{1}/compose".format(
+            compose, respin)
+        url = '/'.join((fedfind.const.NIGHTLY_BASE, path))
+        super(BikeshedModularNightly, self).__init__(
+            url, release="Bikeshed", milestone='', compose=compose,
+            respin=respin, typ="nightly", cid=cid)
+
+    @property
+    def expected_images(self):
+        return (
+            ('docker_base', 'docker', 'x86_64'),
+            ('server', 'dvd', 'arm'),
+            ('server', 'dvd', 'x86_64'),
+            ('server', 'qcow2', 'x86_64'),
+            ('server', 'raw-xz', 'x86_64'),
+        )
+
+
 class BranchedNightly(Pungi4Release):
     """Branched "nightly" (bit of a misnomer, now) composes."""
     def __init__(self, release, compose, respin=0, cid=None):
@@ -1090,6 +1206,32 @@ class BranchedNightly(Pungi4Release):
         return tuple(imgs)
 
 
+class BranchedModularNightly(Pungi4Release):
+    """Branched modular "nightly" (bit of a misnomer, now) composes."""
+    def __init__(self, release, compose, respin=0, cid=None):
+        release = str(release)
+        compose = str(compose)
+        respin = str(respin)
+        path = "Fedora-Modular-{0}-{1}.n.{2}/compose".format(
+            release, compose, respin)
+        url = '/'.join((fedfind.const.NIGHTLY_BASE, path))
+        super(BranchedModularNightly, self).__init__(
+            url, release=release, milestone="Branched", compose=compose,
+            respin=respin, typ="nightly", cid=cid)
+        self._release = release
+        self._milestone = "Branched"
+
+    @property
+    def expected_images(self):
+        return (
+            ('docker_base', 'docker', 'x86_64'),
+            ('server', 'dvd', 'arm'),
+            ('server', 'dvd', 'x86_64'),
+            ('server', 'qcow2', 'x86_64'),
+            ('server', 'raw-xz', 'x86_64'),
+        )
+
+
 class AtomicNightly(Pungi4Release):
     """Two-week Atomic nightly composes. Note, these are usually
     'production' composes in productmd terms, even though they're
@@ -1110,16 +1252,16 @@ class AtomicNightly(Pungi4Release):
     @property
     def expected_images(self):
         """See abstract class docstring for information on what this
-        is. These nightlies are pretty odd, and only Cloud images are
-        built, so we do not take the 'universal' definition from the
-        Release class, but we do 'expect' some non-blocking images,
-        because I think people interested in these nightlies probably
-        want to make sure all the Cloud images built.
+        is. These nightlies include only Atomic images, so we do not
+        take the 'universal' definition from the Release class, we
+        just 'expect' the important Atomic images. Historical note:
+        these composes included Cloud images for some time, but this
+        stopped around early 2017. We don't bother 'expecting' Cloud
+        images for composes from before the change, it just doesn't
+        seem important enough to be worth bothering with.
         """
         return (
-            ('cloud_base', 'raw-xz', 'x86_64'),
-            ('cloud_base', 'vagrant-libvirt', 'x86_64'),
-            ('cloud_base', 'vagrant-virtualbox', 'x86_64'),
+            ('atomic', 'qcow2', 'x86_64'),
             ('atomic', 'raw-xz', 'x86_64'),
             ('atomic', 'vagrant-libvirt', 'x86_64'),
             ('atomic', 'vagrant-virtualbox', 'x86_64'),
@@ -1127,6 +1269,39 @@ class AtomicNightly(Pungi4Release):
         )
 
 
+class CloudNightly(Pungi4Release):
+    """Cloud nightly composes for the current stable release (like
+    the Atomic nightly composes, except they only build Cloud images).
+    As with AtomicNightly, these are 'production' type composes, even
+    though they happen nightly.
+    """
+    def __init__(self, release, compose, respin=0, cid=None):
+        release = str(release)
+        compose = str(compose)
+        respin = str(respin)
+        path = "Fedora-Cloud-{0}-{1}.{2}/compose".format(
+            release, compose, respin)
+        url = '/'.join((fedfind.const.NIGHTLY_BASE, path))
+        super(CloudNightly, self).__init__(
+            url, release=release, milestone="Cloud", compose=compose,
+            respin=respin, typ="production", cid=cid)
+        self._release = release
+
+    @property
+    def expected_images(self):
+        """See abstract class docstring for information on what this
+        is. These nightlies include only Cloud images, so we do not
+        take the 'universal' definition from the Release class, we
+        just 'expect' the intended images.
+        """
+        return (
+            ('cloud_base', 'qcow2', 'x86_64'),
+            ('cloud_base', 'raw-xz', 'x86_64'),
+            ('cloud_base', 'vagrant-libvirt', 'x86_64'),
+            ('cloud_base', 'vagrant-virtualbox', 'x86_64'),
+        )
+
+
 class DockerNightly(Pungi4Release):
     """Docker nightly composes for the current stable release (like
     the Atomic nightly composes, except they only build the Docker
@@ -1148,11 +1323,9 @@ class DockerNightly(Pungi4Release):
     @property
     def expected_images(self):
         """See abstract class docstring for information on what this
-        is. These nightlies are pretty odd, and only Cloud images are
-        built, so we do not take the 'universal' definition from the
-        Release class, but we do 'expect' some non-blocking images,
-        because I think people interested in these nightlies probably
-        want to make sure all the Cloud images built.
+        is. These nightlies include only Docker images, so we do not
+        take the 'universal' definition from the Release class, we
+        just 'expect' the intended images.
         """
         return (
             ('docker_base', 'docker', 'x86_64'),
@@ -1195,6 +1368,35 @@ class Production(Pungi4Release):
             self.release, self.milestone, self.compose, self.respin)
 
 
+class ModularProduction(Pungi4Release):
+    """A modular Pungi 4 'production' compose, on kojipkgs, identified
+    by date and respin (not label). 'compose' is the date. e.g. 27,
+    20171107, 1 for
+    https://kojipkgs.fedoraproject.org/compose/27/Fedora-Modular-27-20171107.1
+    The *same* compose may have been synced to the mirror system and
+    be instantiable as a ModularCompose (see below).
+    """
+    def __init__(self, release, compose, respin, cid=None):
+        release = str(release)
+        compose = str(compose)
+        respin = str(respin)
+        path = "{0}/Fedora-Modular-{0}-{1}.{2}/compose".format(
+            release, compose, respin)
+        url = '/'.join((fedfind.const.NIGHTLY_BASE, path))
+        super(ModularProduction, self).__init__(url, release=release, cid=cid)
+        # We do not set compose or milestone as we want those to be
+        # parsed from the label - see class docstring.
+
+    @property
+    def version(self):
+        """Use release milestone compose.respin, but don't set
+        _version in init as that would cause remote trips, do it here
+        as a property.
+        """
+        return "{0} Modular {1} {2}.{3}".format(
+            self.release, self.milestone, self.compose, self.respin)
+
+
 class Compose(Pungi4Release):
     """A candidate compose that has been synced to the staging tree
     on the mirrors. The exact same compose may be available from
@@ -1223,6 +1425,31 @@ class Compose(Pungi4Release):
         """Assume mirrored composes are always FINISHED."""
         return "FINISHED"
 
+
+class ModularCompose(Pungi4Release):
+    """A modular candidate compose that has been synced to the staging
+    tree on the mirrors. The exact same compose may be available from
+    kojipkgs as a ModularProduction.
+    """
+    def __init__(self, release, milestone, compose, respin, cid=None):
+        release = str(release)
+        milestone = str(milestone)
+        compose = str(compose)
+        respin = str(respin)
+        label = "{0}-{1}.{2}".format(milestone, compose, respin)
+        version = "{0} Modular {1} {2}.{3}".format(release, milestone, compose, respin)
+        url = '{0}/alt/stage/{1}_Modular_{2}'.format(
+            fedfind.const.HTTPS_DL, release, label)
+        super(ModularCompose, self).__init__(
+            url, release=release, milestone=milestone, compose=compose,
+            respin=respin, label=label, typ="production", version=version,
+            cid=cid)
+
+    @property
+    def status(self):
+        """Assume mirrored composes are always FINISHED."""
+        return "FINISHED"
+
 ## PUNGI 4 MIRROR CLASSES ##
 
 
@@ -1781,7 +2008,7 @@ class RespinRelease(MirrorRelease):
         # if we got this far, we already know we exist
         self._exists = True
         # custom fake CID for these releases
-        self._fakecid = "FedoraRespin-{0}-{1}.0".format(self.release, self.compose)
+        self._fakecid = "FedoraRespin-{0}-updates-{1}.0".format(self.release, self.compose)
 
     def __repr__(self):
         return "{0}()".format(self.__class__)
@@ -1840,3 +2067,5 @@ class RespinRelease(MirrorRelease):
         on what packages they contain.
         """
         return {}
+
+# vim: set textwidth=100 ts=8 et sw=4:
