@@ -17,6 +17,9 @@
 #
 # Author: Adam Williamson <awilliam@redhat.com>
 
+# these are all kinda inappropriate for pytest patterns
+# pylint: disable=old-style-class, no-init, protected-access, no-self-use, unused-argument
+
 """Tests for release.py."""
 
 from __future__ import unicode_literals
@@ -152,45 +155,46 @@ F18PACKAGEHTMLB = """
 # Fake PDC response for get_package_nvras testing.
 
 PDCPACKAGES = [
-        {
-            "id": 222481,
-            "name": "anaconda",
-            "version": "24.13",
-            "epoch": 0,
-            "release": "1.fc24",
-            "arch": "src",
-            "srpm_name": "anaconda",
-            "srpm_nevra": "anaconda-0:24.13-1.fc24.src",
-            "filename": "anaconda-24.13-1.fc24.src.rpm",
-        },
-        {
-            "id": 120399,
-            "name": "bash",
-            "version": "4.3.42",
-            "epoch": 0,
-            "release": "4.fc24",
-            "arch": "src",
-            "srpm_name": "bash",
-            "srpm_nevra": "bash-0:4.3.42-4.fc24.src",
-            "filename": "bash-4.3.42-4.fc24.src.rpm",
-        },
-        {
-            "id": 222118,
-            "name": "amanda",
-            "version": "3.3.8",
-            "epoch": 0,
-            "release": "1.fc24",
-            "arch": "src",
-            "srpm_name": "amanda",
-            "srpm_nevra": "amanda-0:3.3.8-1.fc24.src",
-            "filename": "amanda-3.3.8-1.fc24.src.rpm",
-        }
+    {
+        "id": 222481,
+        "name": "anaconda",
+        "version": "24.13",
+        "epoch": 0,
+        "release": "1.fc24",
+        "arch": "src",
+        "srpm_name": "anaconda",
+        "srpm_nevra": "anaconda-0:24.13-1.fc24.src",
+        "filename": "anaconda-24.13-1.fc24.src.rpm",
+    },
+    {
+        "id": 120399,
+        "name": "bash",
+        "version": "4.3.42",
+        "epoch": 0,
+        "release": "4.fc24",
+        "arch": "src",
+        "srpm_name": "bash",
+        "srpm_nevra": "bash-0:4.3.42-4.fc24.src",
+        "filename": "bash-4.3.42-4.fc24.src.rpm",
+    },
+    {
+        "id": 222118,
+        "name": "amanda",
+        "version": "3.3.8",
+        "epoch": 0,
+        "release": "1.fc24",
+        "arch": "src",
+        "srpm_name": "amanda",
+        "srpm_nevra": "amanda-0:3.3.8-1.fc24.src",
+        "filename": "amanda-3.3.8-1.fc24.src.rpm",
+    }
 ]
 
 
 class FakeRelease(object):
     """Release stub used for testing nightly respin guessing. Gives
-    the magic 'good' values for exists and status if respin is 3.
+    the magic 'good' values for exists and status if respin is 3 and
+    release is 24 or Rawhide.
     """
     def __init__(self, respin='', *args, **kwargs):
         self.exists = False
@@ -201,20 +205,6 @@ class FakeRelease(object):
             self.status = 'FINISHED'
 
 
-class FakeReleaseExists(object):
-    """A fake release which always exists."""
-    def __init__(self, *args, **kwargs):
-        self.exists = True
-        self.status = 'FINISHED'
-
-
-class FakeReleaseNotExists(object):
-    """A fake release which never exists."""
-    def __init__(self, *args, **kwargs):
-        self.exists = False
-        self.status = 'DOOMED'
-
-
 class FakeResponsePackages(object):
     """urlopen response stub used for get_package_nvr testing."""
     def __init__(self, url):
@@ -224,25 +214,22 @@ class FakeResponsePackages(object):
         """Returns the appropriate chunk of HTML (see above) if the
         correct URL was given.
         """
-        if self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/core/1/SRPMS/':
+        arcbase = 'https://download.fedoraproject.org/pub/archive/fedora/linux/'
+        branchbase = 'https://kojipkgs.fedoraproject.org/compose/branched/Fedora-24-20160321.n.0/'
+        mapping = {
             # we just pretend 1 is 6, it's easier
-            return FC6PACKAGEHTML
-        elif self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/core/6/source/SRPMS/':
-            return FC6PACKAGEHTML
-        elif self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/releases/16/Everything/source/SRPMS/':
-            return F16PACKAGEHTML
-        elif self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/releases/18/Everything/source/SRPMS/a/':
-            return F18PACKAGEHTMLA
-        elif self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/releases/18/Everything/source/SRPMS/b/':
-            return F18PACKAGEHTMLB
-        elif self.url == 'https://download.fedoraproject.org/pub/archive/fedora/linux/releases/18/Everything/source/SRPMS/f/':
-            return ""
-        elif self.url == 'https://kojipkgs.fedoraproject.org/compose/branched/Fedora-24-20160321.n.0/compose/Everything/source/tree/Packages/a/':
-            return F18PACKAGEHTMLA
-        elif self.url == 'https://kojipkgs.fedoraproject.org/compose/branched/Fedora-24-20160321.n.0/compose/Everything/source/tree/Packages/b/':
-            return F18PACKAGEHTMLB
-        elif self.url == 'https://kojipkgs.fedoraproject.org/compose/branched/Fedora-24-20160321.n.0/compose/Everything/source/tree/Packages/f/':
-            return ""
+            arcbase + 'core/1/SRPMS/': FC6PACKAGEHTML,
+            arcbase + 'core/6/source/SRPMS/': FC6PACKAGEHTML,
+            arcbase + 'releases/16/Everything/source/SRPMS/': F16PACKAGEHTML,
+            arcbase + 'releases/18/Everything/source/SRPMS/a/': F18PACKAGEHTMLA,
+            arcbase + 'releases/18/Everything/source/SRPMS/b/': F18PACKAGEHTMLB,
+            arcbase + 'releases/18/Everything/source/SRPMS/f/': '',
+            branchbase + 'compose/Everything/source/tree/Packages/a/': F18PACKAGEHTMLA,
+            branchbase + 'compose/Everything/source/tree/Packages/b/': F18PACKAGEHTMLB,
+            branchbase + 'compose/Everything/source/tree/Packages/f/': '',
+        }
+        if self.url in mapping:
+            return mapping[self.url]
         else:
             raise ValueError("URL {0} not expected!".format(self.url))
 
@@ -254,69 +241,76 @@ def urlopen_fake_package(url):
 class TestRelease:
     """Tests for release.py."""
 
-    def test_get_release_simple(self):
+    @pytest.mark.parametrize(
+        ("release", "milestone", "compose", "respin", "dist", "expclass",
+         "exprel", "expmile", "expcomp", "expspin"),
+        # rel/mile/comp/respin to pass, expected class, expected rel/mile/comp/respin
+        # None indicates check should not be done
+        [
+            # Archive stable.
+            (15, '', '', None, "Fedora", fedfind.release.ArchiveRelease, '15', None, None, None),
+            # Fedora Core stable (release string).
+            ('6', '', '', None, "Fedora", fedfind.release.CoreRelease, '6', None, None, None),
+            # 'Final' treated as stable.
+            ('6', 'Final', '', None, "Fedora", fedfind.release.CoreRelease, '6', None, None, None),
+            # Milestone.
+            (23, 'Beta', '', None, "Fedora", fedfind.release.Milestone, '23', 'Beta', None, None),
+            # Branched nightly.
+            (23, 'Branched', 'datestr', 1, "Fedora",
+             fedfind.release.BranchedNightly, '23', 'Branched', 'datestr', '1'),
+            # Rawhide nightly.
+            ('Rawhide', '', 'datestr', 2, "Fedora",
+             fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '2'),
+            # Atomic nightly.
+            (24, 'Atomic', 'datestr', 0, "Fedora",
+             fedfind.release.AtomicNightly, '24', 'Atomic', 'datestr', '0'),
+            # Docker nightly.
+            (25, 'Docker', 'datestr', 0, "Fedora",
+             fedfind.release.DockerNightly, '25', 'Docker', 'datestr', '0'),
+            # Cloud nightly.
+            (25, 'Cloud', 'datestr', 0, "Fedora",
+             fedfind.release.CloudNightly, '25', 'Cloud', 'datestr', '0'),
+            # Wikitcms format Rawhide.
+            ('24', 'Rawhide', 'datestr.n.0', None, "Fedora",
+             fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
+            # OFFLINE GUESSES
+            # Date guesses.
+            (23, 'Branched', '', 0, "Fedora",
+             fedfind.release.BranchedNightly, '23', 'Branched', 'datestr', '0'),
+            (27, 'Branched', '', 0, "Fedora-Modular",
+             fedfind.release.BranchedModularNightly, '27', 'Branched', 'datestr', '0'),
+            ('Rawhide', '', '', 0, "Fedora", fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
+            ('Rawhide', '', '', 0, "Fedora-Modular",
+             fedfind.release.RawhideModularNightly, 'Rawhide', '', 'datestr', '0'),
+            ('Bikeshed', '', '', 0, "Fedora-Modular",
+             fedfind.release.BikeshedModularNightly, 'Bikeshed', '', 'datestr', '0'),
+            (24, 'Atomic', '', 0, "Fedora", fedfind.release.AtomicNightly, '24', 'Atomic', 'datestr', '0'),
+            (25, 'Docker', '', 0, "Fedora", fedfind.release.DockerNightly, '25', 'Docker', 'datestr', '0'),
+            (25, 'Cloud', '', 0, "Fedora", fedfind.release.CloudNightly, '25', 'Cloud', 'datestr', '0'),
+            # Default, check None is handled.
+            (None, None, None, 0, "Fedora", fedfind.release.RawhideNightly, 'Rawhide', '', 'datestr', '0'),
+        ]
+    )
+    def test_get_release_simple(self, release, milestone, compose, respin, dist, expclass, exprel,
+                                expmile, expcomp, expspin):
         """Tests for get_release that require no online guessing or
         checking, hence no mocking.
         """
         datestr = datetime.date.today().strftime('%Y%m%d')
-        rels = (
-            # Archive stable.
-            ((15, '', '', None), (fedfind.release.ArchiveRelease, '15')),
-            # Fedora Core stable (release string).
-            (('6', '', '', None), (fedfind.release.CoreRelease, '6')),
-            # 'Final' treated as stable.
-            (('6', 'Final', '', None), (fedfind.release.CoreRelease, '6')),
-            # Milestone.
-            ((23, 'Beta', '', None), (fedfind.release.Milestone, '23', 'Beta')),
-            # Branched nightly.
-            ((23, 'Branched', datestr, 1), (fedfind.release.BranchedNightly, '23', 'Branched', datestr, '1')),
-            # Rawhide nightly.
-            (('Rawhide', '', datestr, 2), (fedfind.release.RawhideNightly, 'Rawhide', '', datestr, '2')),
-            # Atomic nightly.
-            ((24, 'Atomic', datestr, 0), (fedfind.release.AtomicNightly, '24', 'Atomic', datestr, '0')),
-            # Docker nightly.
-            ((25, 'Docker', datestr, 0), (fedfind.release.DockerNightly, '25', 'Docker', datestr, '0')),
-            # Wikitcms format Rawhide.
-            (('24', 'Rawhide', '{0}.n.0'.format(datestr), None),
-             (fedfind.release.RawhideNightly, 'Rawhide', '', datestr, '0')),
-            # OFFLINE GUESSES
-            # Date guesses.
-            ((23, 'Branched', '', 0), (fedfind.release.BranchedNightly, '23', 'Branched', datestr, '0')),
-            (('Rawhide', '', '', 0), (fedfind.release.RawhideNightly, 'Rawhide', '', datestr, '0')),
-            ((24, 'Atomic', '', 0), (fedfind.release.AtomicNightly, '24', 'Atomic', datestr, '0')),
-            ((25, 'Docker', '', 0), (fedfind.release.DockerNightly, '25', 'Docker', datestr, '0')),
-            # Default, check None is handled.
-            ((None, None, None, 0), (fedfind.release.RawhideNightly, 'Rawhide', '', datestr, '0')),
-        )
-
-        # Test that various get_release() invocations return expected
-        # classes and values. Each rel's first tuple is the release,
-        # milestone, compose values to pass to get_release. Its second
-        # tuple's first value is always the expected class. The next
-        # three values of the second tuple are optional, and they are
-        # the expected release, milestone and compose; each will be
-        # checked if present.
-        for ((release, milestone, compose, respin), expected) in rels:
-            got = fedfind.release.get_release(release, milestone, compose, respin)
-            if not isinstance(got, expected[0]):
-                pytest.fail("Testing {0} {1} {2} got class {3}".format(
-                    release, milestone, compose, got.__class__))
-            if len(expected) > 1:
-                if not got.release == expected[1]:
-                    pytest.fail("Testing {0} {1} {2} got release {3}".format(
-                        release, milestone, compose, got.release))
-            if len(expected) > 2:
-                if not got.milestone == expected[2]:
-                    pytest.fail("Testing {0} {1} {2} got milestone {3}".format(
-                        release, milestone, compose, got.milestone))
-            if len(expected) > 3:
-                if not got.compose == expected[3]:
-                    pytest.fail("Testing {0} {1} {2} got compose {3}".format(
-                        release, milestone, compose, got.compose))
-            if len(expected) > 4:
-                if not got.respin == expected[4]:
-                    pytest.fail("Testing {0} {1} {2} {3} got respin {4}".format(
-                        release, milestone, compose, respin, got.respin))
+        if compose:
+            compose = compose.replace('datestr', datestr)
+        if expcomp:
+            expcomp = expcomp.replace('datestr', datestr)
+        got = fedfind.release.get_release(release, milestone, compose, respin, dist=dist)
+        assert isinstance(got, expclass)
+        if exprel is not None:
+            assert got.release == exprel
+        if expmile is not None:
+            assert got.milestone == expmile
+        if expcomp is not None:
+            assert got.compose == expcomp
+        if expspin is not None:
+            assert got.respin == expspin
 
         # Sanity check tests.
         with pytest.raises(ValueError):
@@ -339,7 +333,7 @@ class TestRelease:
         assert ret.compose == '1'
         assert ret.respin == '1'
 
-    @mock.patch('fedfind.helpers.label_from_cid', return_value="Alpha-1.1")
+    @mock.patch('fedfind.helpers.label_from_cid', return_value="Alpha-1.1", autospec=True)
     @mock.patch('fedfind.release.Compose.exists', True)
     def test_get_release_cid_promote(self, fakelab):
         """Requesting release from a production compose ID with
@@ -353,8 +347,8 @@ class TestRelease:
         assert ret.respin == '1'
         fakelab.assert_called_with('Fedora-24-20160314.0')
 
-    @mock.patch('fedfind.helpers.label_from_cid', return_value="Alpha-1.1")
-    @mock.patch('fedfind.helpers.cid_from_label')
+    @mock.patch('fedfind.helpers.label_from_cid', return_value="Alpha-1.1", autospec=True)
+    @mock.patch('fedfind.helpers.cid_from_label', autospec=True)
     @mock.patch('fedfind.release.Compose.exists', False)
     def test_get_release_cid_promote_fail(self, fakecid, fakelab):
         """This tests a rather snaky path: we call get_release with
@@ -369,6 +363,18 @@ class TestRelease:
         fakelab.assert_called_with('Fedora-24-20160314.0')
         assert fakecid.call_count == 0
 
+    @mock.patch('fedfind.release.ModularProduction.exists', True)
+    def test_get_release_modular_production_cid(self):
+        """Test getting a ModularProduction instance via compose ID."""
+        ret = fedfind.release.get_release(cid='Fedora-Modular-27-20171107.1')
+        assert isinstance(ret, fedfind.release.ModularProduction)
+
+    @mock.patch('fedfind.release.ModularCompose.exists', True)
+    def test_get_release_modular_compose(self):
+        """Test getting a ModularCompose instance."""
+        ret = fedfind.release.get_release(27, 'Beta', 1, 1, dist="Fedora-Modular")
+        assert isinstance(ret, fedfind.release.ModularCompose)
+
     def test_get_release_cid_atomic_new(self):
         """Test that nightly stable Atomic compose IDs are handled.
         They have 'Fedora-Atomic' as the product name instead of
@@ -391,6 +397,17 @@ class TestRelease:
         assert ret.compose == '20170113'
         assert ret.respin == '2'
 
+    def test_get_release_cid_cloud(self):
+        """Test that nightly stable Cloud compose IDs are handled.
+        They have 'Fedora-Cloud' as the product name instead of
+        'Fedora'.
+        """
+        ret = fedfind.release.get_release(cid='Fedora-Cloud-26-20170217.3')
+        assert isinstance(ret, fedfind.release.CloudNightly)
+        assert ret.release == '26'
+        assert ret.compose == '20170217'
+        assert ret.respin == '3'
+
     def test_get_release_cid_fake_stable(self):
         """Getting a release based on the fake CID produced for stable
         releases should return the same release.
@@ -399,17 +416,66 @@ class TestRelease:
         rel2 = fedfind.release.get_release(cid=rel.cid)
         assert rel2.version == rel.version
 
-    def test_get_release_url_cid(self):
-        """Getting a release based on a correct nightly compose URL
+    def test_get_release_url_cid_rawhide(self):
+        """Getting a release based on a correct Rawhide nightly URL
         should work.
         """
-        ret = fedfind.release.get_release(
-            url='https://kojipkgs.fedoraproject.org/compose/rawhide/Fedora-Rawhide-20170126.n.0/compose/')
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/rawhide/Fedora-Rawhide-20170126.n.0/compose/')
+        ret = fedfind.release.get_release(url=url)
         assert isinstance(ret, fedfind.release.RawhideNightly)
         assert ret.release == 'Rawhide'
         assert ret.compose == '20170126'
         assert ret.respin == '0'
 
+    def test_get_release_url_cid_branched(self):
+        """Getting a release based on a correct Branched nightly URL
+        should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/branched/Fedora-27-20170816.n.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.BranchedNightly)
+        assert ret.release == '27'
+        assert ret.compose == '20170816'
+        assert ret.respin == '0'
+
+    def test_get_release_url_cid_rawhide_modular(self):
+        """Getting a release based on a correct Rawhide modular
+        nightly URL should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/Fedora-Modular-Rawhide-20170816.n.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.RawhideModularNightly)
+        assert ret.release == 'Rawhide'
+        assert ret.compose == '20170816'
+        assert ret.respin == '0'
+
+    def test_get_release_url_cid_bikeshed_modular(self):
+        """Getting a release based on a correct Bikeshed modular
+        nightly URL should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/Fedora-Modular-Bikeshed-20171013.n.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.BikeshedModularNightly)
+        assert ret.release == 'Bikeshed'
+        assert ret.compose == '20171013'
+        assert ret.respin == '0'
+
+    def test_get_release_url_cid_branched_modular(self):
+        """Getting a release based on a correct Branched modular
+        nightly URL should work.
+        """
+        url = ('https://kojipkgs.fedoraproject.org/'
+               'compose/Fedora-Modular-27-20170816.n.0/compose/')
+        ret = fedfind.release.get_release(url=url)
+        assert isinstance(ret, fedfind.release.BranchedModularNightly)
+        assert ret.release == '27'
+        assert ret.compose == '20170816'
+        assert ret.respin == '0'
+
     def test_get_release_url_cid_unknown(self):
         """Getting a release based on an unknown URL should return
         a generic Pungi4Release instance, so long as the URL starts
@@ -425,9 +491,9 @@ class TestRelease:
         start with one of our known prefixes should raise a value
         error.
         """
+        url = 'https://foobar.fedoraproject.org/compose/whatthehellisthis/ihavenoidea/compose/'
         with pytest.raises(ValueError):
-            ret = fedfind.release.get_release(
-                url='https://foobar.fedoraproject.org/compose/whatthehellisthis/ihavenoidea/compose/')
+            fedfind.release.get_release(url=url)
 
     @mock.patch('fedfind.release.Compose.exists', True)
     def test_get_release_compose(self):
@@ -443,7 +509,8 @@ class TestRelease:
         assert ret.respin == '1'
         assert ret.label == 'RC-1.1'
 
-    @mock.patch('fedfind.helpers.cid_from_label', return_value='Fedora-24-20160316.3')
+    @mock.patch('fedfind.helpers.cid_from_label', return_value='Fedora-24-20160316.3',
+                autospec=True)
     @mock.patch('fedfind.release.Compose.exists', False)
     def test_get_release_compose_fallback(self, fakecid):
         """A production/candidate compose. This tests the case where
@@ -456,27 +523,35 @@ class TestRelease:
 
     @mock.patch('fedfind.release.ArchiveRelease.exists', False)
     def test_get_stable_current(self):
+        """Ensure that if ArchiveRelease doesn't exist, get_release
+        for a recent stable returns CurrentRelease. Tested number must
+        be higher than the 'just return ArchiveRelease' cutoff, so
+        bump this test when bumping that cutoff.
+        """
         ret = fedfind.release.get_release(23)
         assert isinstance(ret, fedfind.release.CurrentRelease)
         assert ret.release == '23'
 
     @mock.patch('fedfind.release.ArchiveRelease.exists', True)
     def test_get_stable_archive(self):
-        # NOTE: this test is intended to hit the code which checks
-        # whether an ArchiveRelease exists and returns it if so; the
-        # tested number has to be greater than the cutoff which just
-        # returns an ArchiveRelease without checking
+        """Ensure that if ArchiveRelease does exist, get_release for
+        a recent stable returns it. Tested number must be higher than
+        the 'just return ArchiveRelease' cutoff (or else this test
+        will pass even if the feature is broken), so bump this test
+        when bumping that cutoff.
+        """
         ret = fedfind.release.get_release(23)
         assert isinstance(ret, fedfind.release.ArchiveRelease)
         assert ret.release == '23'
 
-    @mock.patch('fedfind.helpers.get_current_release', return_value=23)
+    @mock.patch('fedfind.helpers.get_current_release', return_value=23, autospec=True)
     def test_get_release_guess_release(self, fakecurrent):
-        # Release guessing. We can't test this very hard or we'd have
-        # to keep updating the tests all the goddamned time, or the
-        # test would duplicate the logic of get_current_release and
-        # that seems pointless. But we can at least check it's not
-        # crashing and returns the right class.
+        """Release guessing. We can't test this very hard or we'd have
+        to keep updating the tests all the goddamned time, or the test
+        would duplicate the logic of get_current_release and that
+        seems pointless. But we can at least check it's not crashing
+        and returns the right class.
+        """
         datestr = datetime.date.today().strftime('%Y%m%d')
 
         got = fedfind.release.get_release('', 'Branched', '', 0)
@@ -494,53 +569,100 @@ class TestRelease:
         assert got.compose == datestr
         assert got.release == '23'
 
+        got = fedfind.release.get_release('', 'Cloud', '', 0)
+        assert isinstance(got, fedfind.release.CloudNightly)
+        assert got.compose == datestr
+        assert got.release == '23'
+
     @mock.patch('fedfind.release.RawhideNightly', FakeRelease)
+    @mock.patch('fedfind.release.RawhideModularNightly', FakeRelease)
+    @mock.patch('fedfind.release.BikeshedModularNightly', FakeRelease)
     @mock.patch('fedfind.release.BranchedNightly', FakeRelease)
+    @mock.patch('fedfind.release.BranchedModularNightly', FakeRelease)
     @mock.patch('fedfind.release.AtomicNightly', FakeRelease)
     @mock.patch('fedfind.release.DockerNightly', FakeRelease)
+    @mock.patch('fedfind.release.CloudNightly', FakeRelease)
     def test_get_release_guess_respin(self):
+        """Test the 'respin guessing' code works (when you request a
+        nightly but don't provide a respin number). Uses the fake
+        release class's feature of 'existing' when the respin is 3.
+        """
         got = fedfind.release.get_release(24, 'Branched', '20160314')
         assert isinstance(got, fedfind.release.BranchedNightly)
         assert got.respin == 3
+        got = fedfind.release.get_release(27, 'Branched', '20170816', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BranchedModularNightly)
+        assert got.respin == 3
         got = fedfind.release.get_release('Rawhide', '', '20160314')
         assert isinstance(got, fedfind.release.RawhideNightly)
         assert got.respin == 3
+        got = fedfind.release.get_release('Rawhide', '', '20170816', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.RawhideModularNightly)
+        assert got.respin == 3
+        got = fedfind.release.get_release('Bikeshed', '', '20171013', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BikeshedModularNightly)
+        assert got.respin == 3
         got = fedfind.release.get_release(24, 'Atomic', '20160628')
         assert isinstance(got, fedfind.release.AtomicNightly)
         assert got.respin == 3
         got = fedfind.release.get_release(24, 'Docker', '20160628')
         assert isinstance(got, fedfind.release.DockerNightly)
         assert got.respin == 3
+        got = fedfind.release.get_release(24, 'Cloud', '20160628')
+        assert isinstance(got, fedfind.release.CloudNightly)
+        assert got.respin == 3
 
-    @mock.patch('fedfind.release.BranchedNightly', FakeReleaseExists)
-    @mock.patch('fedfind.release.AtomicNightly', FakeReleaseExists)
+    @mock.patch.object(fedfind.release.BranchedNightly, 'exists', True)
+    @mock.patch.object(fedfind.release.BranchedNightly, 'status', 'FINISHED')
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'exists', True)
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'status', 'FINISHED')
+    @mock.patch.object(fedfind.release.AtomicNightly, 'exists', True)
+    @mock.patch.object(fedfind.release.AtomicNightly, 'status', 'FINISHED')
     def test_get_release_guess_nightly_branched_exists(self):
         """Test nightly type guessing: if we specify a date and a
         release number, but no milestone, fedfind should guess between
         BranchedNightly and AtomicNightly. If Branched exists, we
-        should return it.
+        should return it. Also check this path works OK for modular
+        Branched.
         """
         got = fedfind.release.get_release(25, '', '20161006')
         assert isinstance(got, fedfind.release.BranchedNightly)
         got = fedfind.release.get_release(25, '', '20161006', 3)
         assert isinstance(got, fedfind.release.BranchedNightly)
+        got = fedfind.release.get_release(27, '', '20170816', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BranchedModularNightly)
+        got = fedfind.release.get_release(27, '', '20170816', 3, dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BranchedModularNightly)
 
-    @mock.patch('fedfind.release.BranchedNightly', FakeReleaseNotExists)
-    @mock.patch('fedfind.release.AtomicNightly', FakeReleaseExists)
-    def test_get_release_guess_nightly_branched_exists(self):
+    @mock.patch.object(fedfind.release.BranchedNightly, 'exists', False)
+    @mock.patch.object(fedfind.release.BranchedNightly, 'status', 'DOOMED')
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'exists', False)
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'status', 'DOOMED')
+    @mock.patch.object(fedfind.release.AtomicNightly, 'exists', True)
+    @mock.patch.object(fedfind.release.AtomicNightly, 'status', 'FINISHED')
+    def test_get_release_guess_nightly_branched_notexists(self):
         """Test nightly type guessing: if we specify a date and a
         release number, but no milestone, fedfind should guess between
         BranchedNightly and AtomicNightly. If Branched doesn't exist
-        but Atomic does, we should return Atomic.
+        but Atomic does, we should return Atomic. Also check this path
+        works OK for modular Branched.
         """
         got = fedfind.release.get_release(25, '', '20161006')
         assert isinstance(got, fedfind.release.AtomicNightly)
         got = fedfind.release.get_release(25, '', '20161006', 3)
         assert isinstance(got, fedfind.release.AtomicNightly)
+        got = fedfind.release.get_release(27, '', '20170816', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.AtomicNightly)
+        got = fedfind.release.get_release(27, '', '20170816', 3, dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.AtomicNightly)
 
-    @mock.patch('fedfind.release.BranchedNightly', FakeReleaseNotExists)
-    @mock.patch('fedfind.release.AtomicNightly', FakeReleaseNotExists)
-    def test_get_release_guess_nightly_branched_exists(self):
+    @mock.patch.object(fedfind.release.BranchedNightly, 'exists', False)
+    @mock.patch.object(fedfind.release.BranchedNightly, 'status', 'DOOMED')
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'exists', False)
+    @mock.patch.object(fedfind.release.BranchedModularNightly, 'status', 'DOOMED')
+    @mock.patch.object(fedfind.release.AtomicNightly, 'exists', False)
+    @mock.patch.object(fedfind.release.AtomicNightly, 'status', 'DOOMED')
+    def test_get_release_guess_nightly_neitherexists(self):
         """Test nightly type guessing: if we specify a date and a
         release number, but no milestone, fedfind should guess between
         BranchedNightly and AtomicNightly. If neither exists, we should
@@ -550,6 +672,10 @@ class TestRelease:
         assert isinstance(got, fedfind.release.BranchedNightly)
         got = fedfind.release.get_release(25, '', '20161006', 3)
         assert isinstance(got, fedfind.release.BranchedNightly)
+        got = fedfind.release.get_release(27, '', '20170816', dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BranchedModularNightly)
+        got = fedfind.release.get_release(27, '', '20170816', 3, dist='Fedora-Modular')
+        assert isinstance(got, fedfind.release.BranchedModularNightly)
 
     def test_get_release_respinrelease(self, http):
         """Test get_release for RespinRelease class. We use the fake
@@ -565,10 +691,13 @@ class TestRelease:
         # same release
         got2 = fedfind.release.get_release(cid=got.cid)
         assert got.version == got2.version
+        # getting a release with the old form of fake compose ID also
+        got3 = fedfind.release.get_release(cid='FedoraRespin-24-20161120.0')
+        assert got.version == got3.version
         # getting a release from the URL should get the same release
-        got3 = fedfind.release.get_release(
+        got4 = fedfind.release.get_release(
             url='https://dl.fedoraproject.org/pub/alt/live-respins/')
-        assert got3.version == got2.version
+        assert got4.version == got3.version
         # trying to get RespinRelease with a non-matching release
         # or compose should raise an error
         with pytest.raises(ValueError):
@@ -578,28 +707,36 @@ class TestRelease:
         with pytest.raises(ValueError):
             fedfind.release.get_release('25', 'Respin', '20170305')
         with pytest.raises(ValueError):
-            fedfind.release.get_release(cid='FedoraRespin-25-20170305.0')
+            fedfind.release.get_release(cid='FedoraRespin-25-updates-20170305.0')
 
-    def test_get_release_cid_passthru(self):
+    @pytest.mark.parametrize(
+        ("release", "milestone", "compose", "respin", "cid"),
+        [
+            # Branched nightly.
+            (23, 'Branched', 'datestr', 1, 'Fedora-23-datestr.n.1'),
+            # Branched modular nightly.
+            (27, 'Branched', 'datestr', 1, 'Fedora-Modular-27-datestr.n.1'),
+            # Rawhide nightly.
+            ('Rawhide', '', 'datestr', 2, 'Fedora-Rawhide-datestr.n.2'),
+            # Rawhide modular nightly.
+            ('Rawhide', '', 'datestr', 2, 'Fedora-Modular-Rawhide-datestr.n.2'),
+            # Atomic nightly.
+            (24, 'Atomic', 'datestr', 0, 'Fedora-Atomic-24-datestr.0'),
+            # Docker nightly.
+            (25, 'Docker', 'datestr', 0, 'Fedora-Docker-25-datestr.0'),
+            # Cloud nightly.
+            (25, 'Cloud', 'datestr', 0, 'Fedora-Cloud-25-datestr.0'),
+        ]
+    )
+    def test_get_release_cid_passthru(self, release, milestone, compose, respin, cid):
         """Check compose ID is passed through from get_release to the
         Release instance in various cases.
         """
         datestr = datetime.date.today().strftime('%Y%m%d')
-        rels = (
-            # Branched nightly.
-            (23, 'Branched', datestr, 1, 'Fedora-23-{0}.n.1'.format(datestr)),
-            # Rawhide nightly.
-            ('Rawhide', '', datestr, 2, 'Fedora-Rawhide-{0}.n.2'.format(datestr)),
-            # Atomic nightly.
-            (24, 'Atomic', datestr, 0, 'Fedora-Atomic-24-{0}.0'.format(datestr)),
-            # Docker nightly.
-            (25, 'Docker', datestr, 0, 'Fedora-Docker-25-{0}.0'.format(datestr)),
-        )
-        for (release, milestone, compose, respin, cid) in rels:
-            got = fedfind.release.get_release(release, milestone, compose, respin, cid=cid)
-            if not got._cid:
-                pytest.fail("Testing {0} {1} {2}, CID not passed through!".format(
-                    release, milestone, compose, got.__class__))
+        compose = compose.replace('datestr', datestr)
+        cid = cid.replace('datestr', datestr)
+        got = fedfind.release.get_release(release, milestone, compose, respin, cid=cid)
+        assert got._cid == cid
 
     @mock.patch('fedfind.helpers.urlopen_retries', urlopen_fake_package)
     def test_get_package_nvras_fc1(self):
@@ -662,7 +799,7 @@ class TestRelease:
             expected = json.loads(reffh.read())
         for relnum in expected:
             rel = fedfind.release.get_release(relnum)
-            imgs = sorted(rel.all_images, key=lambda x:x['path'])
+            imgs = sorted(rel.all_images, key=lambda x: x['path'])
             assert expected[relnum] == imgs
             # we have to force the release to 'exist' or else
             # check_expected bails
@@ -709,8 +846,22 @@ class TestRelease:
             'anaconda': 'anaconda-18.37.11-1.fc18.src',
             'bash': 'bash-4.2.39-3.fc18.src',
             'fakepackage': ''
+        }
+#        assert pkgs == {
 #            'amanda': 'amanda-0:3.3.8-1.fc24.src',
 #            'anaconda': 'anaconda-0:24.13-1.fc24.src',
 #            'bash': 'bash-0:4.3.42-4.fc24.src',
-        }
-        #fakepdc.assert_called_with('rpms', [('compose', 'Fedora-24-20160321.n.0'), ('arch', 'src'), ('name', 'amanda'), ('name', 'anaconda'), ('name', 'bash'), ('name', 'fakepackage')])
+#        }
+#        fakepdc.assert_called_with(
+#            'rpms',
+#            [
+#                ('compose', 'Fedora-24-20160321.n.0'),
+#                ('arch', 'src'),
+#                ('name', 'amanda'),
+#                ('name', 'anaconda'),
+#                ('name', 'bash'),
+#                ('name', 'fakepackage')
+#            ]
+#        )
+
+# vim: set textwidth=100 ts=8 et sw=4:
